/**
 * @typedef {import("./layers").EoxLayer} EoxLayer
 * @typedef {import("./types").EOxAnimationOptions} EOxAnimationOptions
 * @typedef {import("./types").ControlDictionary} ControlDictionary
 * @typedef {import("./types").ConfigObject} ConfigObject
 * @typedef {import("./types").ProjectionLike} ProjectionLike
 * @typedef {import("./types").AnyLayerWithSource} AnyLayerWithSource
 * */
/**
 * The `eox-map` is a wrapper for the library [OpenLayers](https://openlayers.org/) with additional features and helper functions.
 *
 * Basic usage:
 *
 * ```
 * import "@eox/map"
 *
 * <eox-map [...]></eox-map>
 * ```
 *
 * Some basic layers, sources and formats are included in the default bundle, for advanced usage it is
 * required to import the `advancedLayersAndSources` plugin.
 *
 * Included in the base bundle:
 * - Formats: `GeoJSON`, `MVT`
 * - Layers: `Group`, `Image`, `Tile`, `Vector`, `VectorTile`
 * - Sources: `ImageWMS`, `OSM`, `Tile`, `TileWMS`, `Vector`, `VectorTile`, `WMTS`, `XYZ`
 *
 * In order to use the rest of the layers and sources provided by OpenLayers, import the plugin as well:
 *
 * ```
 * import "@eox/map/src/plugins/advancedLayersAndSources"
 * import "@eox/map"
 *
 * <eox-map [...]></eox-map>
 * ```
 * Included in the advanced plugin bundle:
 * - Layers:
 *   - All OpenLayers layer types
 *   - [`STAC`](https://github.com/m-mohr/ol-stac)
 * - Sources:
 *   - All OpenLayers source types
 *   - [`WMTSCapabilities`](https://github.com/EOX-A/EOxElements/tree/main/elements/map/src/custom/sources/WMTSCapabilities.ts)
 * - Reprojection through [proj4](https://github.com/proj4js/proj4js)
 *
 * @element eox-map
 */
export class EOxMap extends LitElement {
    static get properties(): {
        center: {
            attribute: boolean;
            type: ArrayConstructor;
        };
        layers: {
            attribute: boolean;
            type: ArrayConstructor;
        };
        preventScroll: {
            attribute: string;
            type: BooleanConstructor;
        };
        config: {
            attribute: boolean;
            type: ObjectConstructor;
        };
        sync: {
            attribute: string;
            type: StringConstructor;
        };
        projection: {
            attribute: string;
            type: StringConstructor;
        };
        map: {
            attribute: boolean;
            state: boolean;
        };
        intersections: {
            attribute: boolean;
            state: boolean;
            type: ObjectConstructor;
        };
        selectInteractions: {
            attribute: boolean;
            state: boolean;
            type: ObjectConstructor;
        };
        mapControls: {
            attribute: boolean;
            state: boolean;
            type: ObjectConstructor;
        };
    };
    /**
     * The OpenLayers map instance.
     *
     * @type {import("ol/Map").default}
     */
    map: import("ol/Map").default;
    /**
     * Object to store various map interactions (e.g., drag, zoom).
     *
     * @type {Object.<string, import("ol/interaction/Interaction").default>}
     */
    interactions: {
        [x: string]: import("ol/interaction").Interaction;
    };
    /**
     * Object to store selection interactions for the map.
     *
     * @type {Object.<string, import("./helpers/select").EOxSelectInteraction>}
     */
    selectInteractions: {
        [x: string]: import("./helpers").EOxSelectInteraction;
    };
    /**
     * Object to store map controls (e.g., custom buttons, geolocation).
     *
     * @type {Object.<string, import("ol/control/Control").default>}
     */
    mapControls: {
        [x: string]: import("ol/control/Control").default;
    };
    /**
     * Sets the center of the map. If the new center is valid, updates the map's view.
     *
     * @param {Array<number>} center - The new center coordinates [x, y].
     */
    set center(center: Array<number>);
    /**
     * Gets the current center coordinates of the map.
     *
     * @returns {Array<number>} The current center of the map.
     */
    get center(): Array<number>;
    /**
     * Gets the current center of the map in longitude and latitude.
     *
     * @returns {Array<number>} The geographic center [longitude, latitude].
     */
    get lonLatCenter(): Array<number>;
    /**
     * Gets the current extent of the map in longitude and latitude.
     *
     * @returns {Array<number>} The geographic extent [minLon, minLat, maxLon, maxLat].
     */
    get lonLatExtent(): Array<number>;
    /**
     * Sets the zoom level of the map and animates the change.
     *
     * @param {number} zoom - The new zoom level.
     */
    set zoom(zoom: number);
    /**
     * Gets the current zoom level of the map.
     *
     * @returns {number} The current zoom level.
     */
    get zoom(): number;
    /**
     * Sets the zoom extent of the map.
     *
     * @param {import("ol/extent").Extent} extent - The new zoom extent.
     */
    set zoomExtent(extent: import("ol/extent").Extent);
    /**
     * Sets the controls for the map.
     *
     * @param {ControlDictionary} controls - An array of control configurations.
     */
    set controls(controls: ControlDictionary);
    /**
     * Gets the current map controls.
     *
     * @returns {ControlDictionary} The current controls applied to the map.
     */
    get controls(): ControlDictionary;
    /**
     * Sets the layers for the map.
     *
     * @param {Array<EoxLayer>} layers - An array of layer configurations.
     */
    set layers(layers: Array<EoxLayer>);
    /**
     * Gets the current layers of the map.
     *
     * @returns {Array<EoxLayer>} The current layers applied to the map.
     */
    get layers(): Array<EoxLayer>;
    /**
     * Enables or disables scroll interactions on the map.
     *
     * @param {boolean} preventScroll - Whether to prevent scroll interactions.
     */
    set preventScroll(preventScroll: boolean);
    /**
     * Gets the current scroll interaction state.
     *
     * @returns {boolean} `true` if scroll interactions are prevented, `false` otherwise.
     */
    get preventScroll(): boolean;
    /**
     * Sets the configuration for the map.
     *
     * @param {ConfigObject} config - The configuration object.
     */
    set config(config: ConfigObject);
    /**
     * Gets the current configuration of the map.
     *
     * @returns {ConfigObject} The map's configuration object.
     */
    get config(): ConfigObject;
    /**
     * Sets animation options for map view changes.
     *
     * @param {EOxAnimationOptions} animationOptions - The animation options.
     */
    set animationOptions(animationOptions: EOxAnimationOptions);
    /**
     * Gets the current animation options.
     *
     * @returns {EOxAnimationOptions} The current animation options for the map.
     */
    get animationOptions(): EOxAnimationOptions;
    /**
     * Sets the map's projection.
     *
     * @param {ProjectionLike} projection - The projection code (e.g., "EPSG:3857").
     */
    set projection(projection: ProjectionLike);
    /**
     * Gets the current map projection.
     *
     * @returns {ProjectionLike} The map's projection code.
     */
    get projection(): ProjectionLike;
    /**
     * Sets the sync state for the map.
     *
     * @param {string} sync - The ID of the map to sync with.
     */
    set sync(sync: string);
    /**
     * Gets the current sync state of the map.
     *
     * @returns {string} The ID of the map that this map is synced with.
     */
    get sync(): string;
    /**
     * Adds or updates a layer on the map.
     *
     * @param {EoxLayer} json - The layer configuration in JSON format.
     * @returns {Object} The added or updated layer.
     */
    addOrUpdateLayer(json: EoxLayer): any;
    /**
     * Removes an interaction from the map by its ID.
     *
     * @param {string | number} id - The ID of the interaction to remove.
     */
    removeInteraction(id: string | number): void;
    /**
     * Removes a select interaction from the map by its ID.
     *
     * @param {string | number} id - The ID of the select interaction to remove.
     */
    removeSelect(id: string | number): void;
    /**
     * Removes a control from the map by its ID.
     *
     * @param {string} id - The ID of the control to remove.
     */
    removeControl(id: string): void;
    /**
     * Retrieves a layer from the map by its ID.
     *
     * @param {string} layerId - The ID of the layer to retrieve.
     * @returns {AnyLayerWithSource} The layer object.
     */
    getLayerById(layerId: string): AnyLayerWithSource;
    /**
     * Lifecycle method called after the component's first update.
     * Sets up initial configurations like zoom extent.
     */
    firstUpdated(): void;
    /**
     * Parses a feature from the input data.
     *
     * @type {Function}
     */
    parseFeature: Function;
    /**
     * Parses text into a feature.
     */
    parseTextToFeature: typeof parseTextToFeature;
    /**
     * Registers a projection from an EPSG code.
     */
    registerProjectionFromCode: typeof registerProjectionFromCode;
    /**
     * Registers a custom projection.
     */
    registerProjection: typeof registerProjection;
    /**
     * Retrieves all layers in a flat array.
     */
    getFlatLayersArray: typeof getFlatLayersArray;
    /**
     * Transforms coordinates between different projections.
     */
    transform: typeof transform;
    /**
     * Transforms the extent between different projections.
     */
    transformExtent: typeof transformExtent;
    /**
     * Applies a buffer around an extent.
     */
    buffer: typeof buffer;
    render(): import("lit-html").TemplateResult<1>;
    #private;
}
export type EoxLayer = import("./layers").EoxLayer;
export type EOxAnimationOptions = import("./types").EOxAnimationOptions;
export type ControlDictionary = import("./types").ControlDictionary;
export type ConfigObject = import("./types").ConfigObject;
export type ProjectionLike = import("./types").ProjectionLike;
export type AnyLayerWithSource = import("./types").AnyLayerWithSource;
import { LitElement } from "lit";
import { parseTextToFeature } from "./helpers";
import { registerProjectionFromCode } from "./helpers";
import { registerProjection } from "./helpers";
import { getFlatLayersArray } from "./helpers";
import { transform } from "./helpers";
import { transformExtent } from "./helpers";
import { buffer } from "ol/extent";
//# sourceMappingURL=main.d.ts.map