/**
 * @typedef {import("../../layers").EoxLayer} EoxLayer
 * @typedef {import("../../types").ControlType} ControlType
 * @typedef {import("../../types").ControlDictionary} ControlDictionary
 * @typedef {import("../../types").SelectOptions} SelectOptions
 * @typedef {import("../../types").ConfigObject} ConfigObject
 * @typedef {import("../../types").ProjectionLike} ProjectionLike
 * */
/**
 * Sets the new center of the map if it differs from the current center.
 *
 * @param {Array<number>} center - The new center coordinates.
 * @param {import("../../main").EOxMap} EOxMap - The map object containing the current map view and projection.
 * @returns {Array<number>|undefined} - The new center coordinates or undefined if unchanged.
 */
export function setCenterMethod(center: Array<number>, EOxMap: import("../../main").EOxMap): Array<number> | undefined;
/**
 * Adjusts the map view to fit the specified extent.
 *
 * @param {import("ol/extent").Extent} extent - The extent to fit the map view to.
 * @param {import("../../main").EOxMap} EOxMap - The map object containing the map view and animation options.
 * @returns {import("ol/extent").Extent} - The extent if valid, otherwise undefined.
 */
export function setZoomExtentMethod(extent: import("ol/extent").Extent, EOxMap: import("../../main").EOxMap): import("ol/extent").Extent;
/**
 * Adds or updates controls on the map, removing any outdated controls.
 *
 * @param {ControlDictionary} controls - New controls to set on the map.
 * @param {ControlDictionary} oldControls - Previously set controls on the map.
 * @param {import("../../main").EOxMap} EOxMap - The map object containing the control management functions.
 * @returns {ControlDictionary} - The new controls.
 */
export function setControlsMethod(controls: ControlDictionary, oldControls: ControlDictionary, EOxMap: import("../../main").EOxMap): ControlDictionary;
/**
 * Adds or updates layers on the map, removing any outdated layers.
 *
 * @param {Array<EoxLayer>} layers - New layers to set on the map.
 * @param {Array<EoxLayer>} oldLayers - Previously set layers on the map.
 * @param {import("../../main").EOxMap} EOxMap - The map object containing layer management functions.
 * @returns {Array<EoxLayer>} - The new layers.
 */
export function setLayersMethod(layers: Array<EoxLayer>, oldLayers: Array<EoxLayer>, EOxMap: import("../../main").EOxMap): Array<EoxLayer>;
/**
 * Enables or disables scroll interactions on the map.
 *
 * @param {boolean} preventScroll - Whether to prevent scroll interactions.
 * @param {import("../../main").EOxMap} EOxMap - The map object containing scroll interaction management functions.
 * @returns {boolean} - The current scroll prevention state.
 */
export function setPreventScrollMethod(preventScroll: boolean, EOxMap: import("../../main").EOxMap): boolean;
/**
 * Sets the map's configuration options.
 *
 * @param {ConfigObject} config - Configuration object containing view, layers, controls, and other properties.
 * @param {import("../../main").EOxMap} EOxMap - The map object to configure.
 * @returns {ConfigObject} - The applied configuration.
 */
export function setConfigMethod(config: ConfigObject, EOxMap: import("../../main").EOxMap): ConfigObject;
/**
 * Sets a new projection for the map, transforming the view and adjusting related properties.
 *
 * @param {ProjectionLike} projection - The new projection code.
 * @param {ProjectionLike} oldProjection - The current projection code.
 * @param {import("../../main").EOxMap} EOxMap - The map object containing the current view and layer management functions.
 * @returns {ProjectionLike} - The new projection code.
 */
export function setProjectionMethod(projection: ProjectionLike, oldProjection: ProjectionLike, EOxMap: import("../../main").EOxMap): ProjectionLike;
/**
 * Synchronizes the map view with another map element.
 *
 * @param {string} sync - The ID of the map element to synchronize with.
 * @param {import("../../main").EOxMap} EOxMap - The map object to sync.
 * @returns {string} - The sync parameter.
 */
export function setSyncMethod(sync: string, EOxMap: import("../../main").EOxMap): string;
export type EoxLayer = import("../../layers").EoxLayer;
export type ControlType = import("../../types").ControlType;
export type ControlDictionary = import("../../types").ControlDictionary;
export type SelectOptions = import("../../types").SelectOptions;
export type ConfigObject = import("../../types").ConfigObject;
export type ProjectionLike = import("../../types").ProjectionLike;
//# sourceMappingURL=setters.d.ts.map